MODULE Graphics; 
(* =========================================================================  
   General Graphics Functions  
   Ref: Project Oberon - Wirth & Gutknecht, ACM Press 1992

   Target: Hardware-independent

   (c) 2008-2012 CFB Software   
   http://www.astrobe.com  
   
   ========================================================================= *)

TYPE 
  DrawDotProc* = PROCEDURE (colour, x, y: INTEGER);

VAR
  (* User-supplied, display-dependent function.
     Initially set to DrawDotUndefined *)
  DrawDot: DrawDotProc;
  MaxX, MaxY: INTEGER;

  
PROCEDURE DrawDotUndefined(colour, x, y: INTEGER);
BEGIN
  ASSERT(FALSE, 20)
END DrawDotUndefined;
 

PROCEDURE Line*(colour, x0, y0, x1, y1: INTEGER);
(* Ref: http://en.wikipedia.org/wiki/Bresenham's_line_algorithm *)
VAR
  steep: BOOLEAN;
  x, y, deltaX, deltaY, error, ystep: INTEGER;
BEGIN
  steep := ABS(y1 - y0) > ABS(x1 - x0);
  IF steep THEN
    x := x0; x0 := y0; y0 := x;
    x := x1; x1 := y1; y1 := x;
  END;
  IF x0 > x1 THEN
    x := x0; x0 := x1; x1 := x;
    y := y0; y0 := y1; y1 := y
  END;
  deltaX := x1 - x0;
  deltaY := ABS(y1 - y0);
  error := deltaX DIV 2;
  y := y0;
  IF y0 < y1 THEN ystep := 1 ELSE ystep := -1 END;
  FOR x := x0 TO x1 DO 
    IF (x >= 0) & (y >= 0) & (x <= MaxX) & (y <= MaxY) THEN 
      IF steep THEN DrawDot(colour, y, x) ELSE DrawDot(colour, x, y) END
    END;
    error := error - deltaY;
    IF error < 0 THEN
      y := y + ystep;
      error := error + deltaX
    END
  END
END Line;


PROCEDURE Circle*(colour, x0, y0, r: INTEGER);
  VAR x, y, u: INTEGER;
BEGIN u := 1 - r; x := r; y := 0;
  WHILE y <= x DO
  	DrawDot(colour, x0+x, y0+y);
  	DrawDot(colour, x0+y, y0+x);
  	DrawDot(colour, x0-y, y0+x);
  	DrawDot(colour, x0-x, y0+y);
  	DrawDot(colour, x0-x, y0-y);
  	DrawDot(colour, x0-y, y0-x);
  	DrawDot(colour, x0+y, y0-x);
  	DrawDot(colour, x0+x, y0-y);
  	IF u < 0 THEN u := u + 2*y+3 ELSE u := u + 2*(y-x)+5; DEC(x) END;
  	INC(y)
  END
END Circle;


PROCEDURE Ellipse*(colour, x0, y0, a, b: INTEGER);
  VAR x, y, y1, aa, bb, d, g, h: INTEGER;
BEGIN 
  aa := a*a; bb := b*b;
  h := (aa DIV 4) - b*aa + bb; g := (9*aa DIV 4) - 3*b*aa + bb; x := 0; y := b;
  WHILE g < 0 DO
  	DrawDot(colour, x0+x, y0+y);
  	DrawDot(colour, x0-x, y0+y);
  	DrawDot(colour, x0-x, y0-y);
  	DrawDot(colour, x0+x, y0-y);
  	IF h < 0 THEN d := (2*x+3)*bb; g := g + d
  	ELSE d := (2*x+3)*bb - 2*(y-1)*aa; g := g + d + 2*aa; DEC(y)
  	END;
  	h := h + d; INC(x)
  END;
  y1 := y; h := (bb DIV 4) - a*bb + aa; x := a; y := 0;
  WHILE y <= y1 DO
  	DrawDot(colour, x0+x, y0+y);
  	DrawDot(colour, x0-x, y0+y);
  	DrawDot(colour, x0-x, y0-y);
  	DrawDot(colour, x0+x, y0-y);
  	IF h < 0 THEN h := h + (2*y+3)*aa ELSE h := h + (2*y+3)*aa - 2*(x-1)*bb; DEC(x) END;
  	INC(y)
  END
END Ellipse;


PROCEDURE Init*(maxX, maxY: INTEGER; dd: DrawDotProc);
BEGIN
  DrawDot := dd;
  MaxX := maxX;
  MaxY := maxY
END Init;


BEGIN
  DrawDot := DrawDotUndefined; 
  MaxX := 0; 
  MaxY := 0
END Graphics.
