MODULE Slide;
(* =========================================================================  
   Display an AutoCad Slide-format File 
  
   Target: Hardware-independent
   
   (c) 2009-2012 CFB Software   
   http://www.astrobe.com  
   
   ========================================================================= *)

  IMPORT Graph, IO, Out, Timer;

  CONST
    offset    = 0FBX; (* -5 *)
    endfile   = 0FCX; (* -4 *)
    endPoint  = 0FEX; (* -2 *)
    newColour = 0FFX; (* -1 *)

  VAR
    c: CHAR;
    colour: INTEGER;
    scaleFactor*: INTEGER;
    zoom: INTEGER;
    hX, hY: INTEGER;
    panX, panY: INTEGER;
    
  PROCEDURE* XScale (x: INTEGER): INTEGER;
  BEGIN
    RETURN (x * zoom DIV scaleFactor) - panX
  END XScale;

  PROCEDURE* YScale (y: INTEGER): INTEGER;
  BEGIN
    RETURN Graph.MaxY - ((y * zoom DIV scaleFactor) - panY)
  END YScale;

  PROCEDURE Draw(zoomm, zoomd, px, py, delay: INTEGER);
  VAR
    drawLine: BOOLEAN;
    end1, end2, fromX, fromY: INTEGER;
    i, count: INTEGER;
    lastX, lastY: INTEGER;
    offset1, offset2, offset3, offset4: INTEGER;
    toX, toY: INTEGER;
  BEGIN
    Graph.Init;
    IO.Reset();
    IO.ReadHeader(hX, hY);
    IF hX > hY THEN 
      scaleFactor := (hX DIV zoomm); zoom := zoomd * Graph.MaxX 
    ELSE 
      scaleFactor := (hY DIV zoomm); zoom := zoomd * Graph.MaxY 
    END;
    panX := px;
    panY := py;
    c := IO.ReadType();
    lastX := 0;
    lastY := 0;
    i := 0;
    count := 0;
    REPEAT
      drawLine := TRUE;
      IF c = offset THEN 
          IO.GetOffset(offset1, offset2, offset3, offset4);
          fromX := lastX + offset1;
          fromY := lastY + offset2;
          toX := lastX + offset3;
          toY := lastY + offset4;
          lastX := fromX;
          lastY := fromY;
      ELSIF c = endPoint THEN 
          fromX := lastX;
          fromY := lastY;
          IO.GetEndpoint(end1, end2);
          toX := lastX + end1;
          toY := lastY + end2;
          lastX := toX;
          lastY := toY;
      ELSIF c = newColour THEN 
          colour := IO.GetColour();
          drawLine := FALSE
      ELSE
          IO.GetVector(fromX, fromY, toX, toY);
          lastX := fromX;
          lastY := fromY
      END;
      IF drawLine THEN
        Graph.Line(XScale(fromX), YScale(fromY), XScale(toX), YScale(toY), colour);
        IF delay > 0 THEN Timer.MSecDelay(delay) END;
      END;
      c := IO.ReadType();
      INC(count)
    UNTIL (c = endfile)
  END Draw;

  PROCEDURE Display*(name: ARRAY OF CHAR; delay: INTEGER);
  BEGIN
     IF IO.Open(name) THEN
      Draw(1, 1, 0, -40, delay)
    ELSE
      Out.String("name "); Out.String("not found"); Out.Ln
    END 
  END Display;

END Slide.
